@@ -18,11 +18,13 @@ class JSONWebToken:
             for the token.
     """
     __module__ = 'unimatrix.ext.jose'
-    Expired = exceptions.Expired
-    InsufficientScope = exceptions.InsufficientScope
-    InvalidAudience = exceptions.InvalidAudience
-    InvalidIssuer = exceptions.InvalidIssuer
-    MalformedToken = exceptions.MalformedToken
+    Expired             = exceptions.Expired
+    InsufficientScope   = exceptions.InsufficientScope
+    InvalidAudience     = exceptions.InvalidAudience
+    InvalidIssuer       = exceptions.InvalidIssuer
+    MalformedToken      = exceptions.MalformedToken
+    MissingClaims       = exceptions.MissingClaims
+    NotEffective        = exceptions.NotEffective
 
     @property
     def aud(self) -> set:
@@ -47,11 +49,37 @@ class JSONWebToken:
         except ValueError:
             raise self.MalformedToken
 
+    @property
+    def iat(self) -> int:
+        """Return the issued at date/time, in seconds since the UNIX epoch,
+        as specified by the `iat` claim.
+        """
+        if self.claims.get('iat') is None:
+            return None
+
+        try:
+            return int(self.claims['iat'])
+        except ValueError:
+            raise self.MalformedToken
+
     @property
     def iss(self) -> str:
         """Return the issues as specified by the `iss` claim."""
         return self.claims.get('iss')
 
+    @property
+    def nbf(self) -> int:
+        """Return the not before date/time, in seconds since the UNIX epoch,
+        as specified by the `nbf` claim.
+        """
+        if self.claims.get('nbf') is None:
+            return None
+
+        try:
+            return int(self.claims['nbf'])
+        except ValueError:
+            raise self.MalformedToken
+
     @functools.cached_property
     def scopes(self) -> set:
         """Return the scopes specified by the token, either as an array
@@ -111,18 +139,45 @@ class JSONWebToken:
         audience: list = None,
         issuers: list = None,
         scope: list = None,
-        raise_exception: bool = True
+        raise_exception: bool = True,
+        max_age: int = None,
+        required: set = None
     ) -> bool:
         """Verify if the JWT is not expired, issued by a trusted party, and
         contains the correct scope.
+
+        Args:
+            audience (set): the set of audiences that are accepted.
+            issuers (set): the set of issuers that are accepted.
+            scope (set): the set of scopes that are accepted.
+            raise_exception (bool): indicates if an exception must be raised
+                on verification failures, instead of returning a boolean.
+                Default is ``True``.
+            max_age (int): the maximum age of the token, in seconds. Providing
+                `max_age` on a token that has not set the ``iat`` claim,
+                always causes a failure.
+            required (set): the set of claims that must be present on the
+                token.
+
+        Returns:
+            :class:`bool`
         """
-        self.verify_expired(raise_exception=raise_exception)
+        now = int(time.time())
+        self.verify_expired(now=now, raise_exception=raise_exception)
+        self.verify_not_before(now=now, raise_exception=raise_exception)
+        if max_age is not None:
+            self.verify_max_age(
+                max_age, now=now,
+                raise_exception=raise_exception
+            )
         if audience is not None:
             self.verify_audience(audience, raise_exception=raise_exception)
         if issuers is not None:
             self.verify_issuer(issuers, raise_exception=raise_exception)
         if scope is not None:
             self.verify_scopes(scope, raise_exception=raise_exception)
+        if required is not None:
+            self.verify_presence(required)
 
     def verify_expired(self, now=None, raise_exception=True):
         """Verifies that the JSON Web Token is not expired, based on the
@@ -147,6 +202,39 @@ class JSONWebToken:
             raise self.InvalidIssuer(self, issuers)
         return is_valid
 
+    def verify_max_age(self, max_age, now=None, raise_exception=True):
+        """Verifies that the token does not exceed the maximum age defined.
+        If there is no ``iat`` claim, then verification always fails.
+        """
+        now = now or int(time.time())
+        if self.iat is None or ((now - self.iat) > max_age):
+            raise self.Expired
+
+    def verify_not_before(self, now=None, raise_exception=True):
+        """Verifies that the JSON Web Token is not used before it is allowed,
+        based on the `nbf` claim.
+        """
+        if self.nbf is None:
+            return True
+
+        if now is None:
+            now = int(time.time())
+        is_valid = self.nbf <= now
+        if not is_valid and raise_exception:
+            raise self.NotEffective(self, now)
+        return is_valid
+
+    def verify_presence(self, claims: set, raise_exception=True) -> bool:
+        """Verifies that the set of claims is present on the token."""
+        missing = set(claims) - set(self.claims.keys())
+        if missing and raise_exception:
+            raise self.MissingClaims(
+                detail=(
+                    f"These claims were missing: {', '.join(missing)}"
+                )
+            )
+        return not bool(missing)
+
     def verify_scopes(self, required_scopes, raise_exception=True):
         """Verifies that the scopes are equal or greater than
         `required_scopes`.
